<?php

namespace Drupal\openstack_provider\Service;

use Drupal\Core\Entity\EntityInterface;
use Drupal\facade\Entity\TenantInterface;
use Drupal\cloud\Entity\CloudConfig;
use Drupal\openstack_provider\Entity\OpenStackTenantProjectInterface;

/**
 * OpenStack Provider interface.
 */
interface OpenStackProviderServiceInterface {

  /**
   * Takes a yaml file location, parses it and fetches the values.
   *
   * @param string $yaml
   *   Yaml file location relative to openstack_provider module.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   Entity interface.
   *
   * @return array
   *   Array of values, ready to pass to AWS CloudFormation.
   */
  public function compileYamlParameters(string $yaml, EntityInterface $entity): array;

  /**
   * Helper to create the OpenStack Resources.
   *
   * @param \Drupal\cloud\Entity\CloudConfig $cloud_config
   *   Cloud config entity.
   * @param \Drupal\facade\Entity\TenantInterface $tenant
   *   The tenant entity to launch.
   * @param string $resource_prefix
   *   Prefix used for resource creation.
   * @param string $project_entity_name
   *   Name for creating OpenStackTenantProject entity.
   *
   * @return \Drupal\openstack_provider\Entity\OpenStackTenantProjectInterface
   *   The created OpenStack tenant project entity.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function createOpenStackResource(CloudConfig $cloud_config, TenantInterface $tenant, string $resource_prefix, string $project_entity_name): OpenStackTenantProjectInterface;

  /**
   * Create a remote cloud config entity from a Tenant entity.
   *
   * The params array is a serialized CloudConfig entity with the correct
   * project_id, username and password to access a specific OpenStack Project.
   * See `OpenStackAddNewTenantProjectForm::submitForm()` for an example on
   * how to serialize the CloudConfig entity.
   *
   * @param \Drupal\facade\Entity\TenantInterface $tenant
   *   The target tenant to create the CloudConfig for.
   * @param array $params
   *   The cloud_config array.
   *
   * @throws \Exception
   *   Throw exception if there is an error.
   */
  public function createRemoteCloudConfig(TenantInterface $tenant, array $params): void;

}
